==> app-0.2.2.1.jar: monkey/ci/git.clj <==
(ns monkey.ci.git
  "Clone and checkout git repos.  This is mostly a wrapper for `clj-jgit`"
  (:require [babashka.fs :as fs]
            [clj-jgit.porcelain :as git]
            [clojure.java.io :as io]
            [clojure.string :as cs]
            [clojure.tools.logging :as log]
            [monkey.ci.utils :as u]))

(defn prepare-ssh-keys
  "Writes any ssh keys in the options to a temp directory and returns their
   file names and key dir to be used by clj-jgit."
  [{:keys [ssh-keys ssh-keys-dir]}]
  (when-let [f (io/file ssh-keys-dir)]
    (when-not (or (.exists f) (.mkdirs f))
      (throw (ex-info "Unable to create ssh key dir" {:dir ssh-keys-dir})))
    (log/debug "Writing" (count ssh-keys) "ssh keys to" f)
    (->> ssh-keys
         (map-indexed (fn [idx r]
                        (let [keys ((juxt :public-key :private-key) r)
                              names (->> [".pub" ""]
                                         (map (partial format "key-%d%s" idx)))
                              paths (map (partial io/file f) names)]
                          (->> (map (fn [n k]
                                      (spit n k)
                                      (fs/set-posix-file-permissions n "rw-------"))
                                    paths keys)
                               (doall))
                          (merge r (zipmap [:public-key-file :private-key-file] (map str names))))))
         (doall)
         (mapv :private-key-file)
         (hash-map :key-dir ssh-keys-dir :name))))

(defn clone
  "Clones the repo at given url, and checks out the given branch.  Writes the
   files to `dir`.  Returns a repo object that can be passed to other functions."
  [{:keys [url branch dir] :as opts}]
  (log/debug "Cloning" url "into" dir)
  ;; Precalculate id config otherwise it gets called multiple times by the `with-identity` macro.
  (let [id-config (merge {:trust-all? true}
                         (prepare-ssh-keys opts))]
    (git/with-identity id-config
      (git/git-clone url
                     :branch branch
                     :dir dir
                     :no-checkout? true))))

(defn checkout [repo id]
  (log/debug "Checking out" id "from repo" repo)
  (git/git-checkout repo {:name id}))

(def origin-prefix "origin/")

(defn- prefix-origin
  "Ensures that the given branch name has the `origin` prefix"
  [b]
  (when b
    (cond->> b
      (not (cs/starts-with? b origin-prefix)) (str origin-prefix))))

(defn clone+checkout
  "Clones the repo, then performs a checkout of the given id"
  [{:keys [branch commit-id ref] :as opts}]
  (let [repo (clone opts)]
    (when-let [id-or-branch (or commit-id ref (prefix-origin branch))]
      (checkout repo id-or-branch))
    repo))

(defn delete-repo
  "Deletes the previously checked out local repo"
  [repo]
  (log/debug "Deleting repository" repo)
  (-> repo
      (.getRepository)
      (.getWorkTree)
      (u/delete-dir)))

==> app-0.2.3-20240130.104423-4.jar: monkey/ci/git.clj <==
(ns monkey.ci.git
  "Clone and checkout git repos.  This is mostly a wrapper for `clj-jgit`"
  (:require [babashka.fs :as fs]
            [clj-jgit.porcelain :as git]
            [clojure.java.io :as io]
            [clojure.string :as cs]
            [clojure.tools.logging :as log]
            [monkey.ci.utils :as u]))

(defn prepare-ssh-keys
  "Writes any ssh keys in the options to a temp directory and returns their
   file names and key dir to be used by clj-jgit."
  [{:keys [ssh-keys ssh-keys-dir]}]
  (when-let [f (io/file ssh-keys-dir)]
    (when-not (or (.exists f) (.mkdirs f))
      (throw (ex-info "Unable to create ssh key dir" {:dir ssh-keys-dir})))
    (log/debug "Writing" (count ssh-keys) "ssh keys to" f)
    (->> ssh-keys
         (map-indexed (fn [idx r]
                        (let [keys ((juxt :public-key :private-key) r)
                              names (->> [".pub" ""]
                                         (map (partial format "key-%d%s" idx)))
                              paths (map (partial io/file f) names)]
                          (->> (map (fn [n k]
                                      (spit n k)
                                      (fs/set-posix-file-permissions n "rw-------"))
                                    paths keys)
                               (doall))
                          (merge r (zipmap [:public-key-file :private-key-file] (map str names))))))
         (doall)
         (mapv :private-key-file)
         (hash-map :key-dir ssh-keys-dir :name))))

(def origin-prefix "origin/")

(defn- prefix-origin
  "Ensures that the given branch name has the `origin` prefix"
  [b]
  (when b
    (cond->> b
      (not (cs/starts-with? b origin-prefix)) (str origin-prefix))))

(defn- opts->branch [{:keys [branch commit-id ref]}]
  (or commit-id ref (prefix-origin branch)))

(defn clone
  "Clones the repo at given url, and checks out the given branch.  Writes the
   files to `dir`.  Returns a repo object that can be passed to other functions."
  [{:keys [url dir] :as opts}]
  ;; Precalculate id config otherwise it gets called multiple times by the `with-identity` macro.
  (let [id-config (merge {:trust-all? true}
                         (prepare-ssh-keys opts))
        branch (opts->branch opts)]
    (git/with-identity id-config
      (log/debug "Cloning" url "and branch" branch "into" dir)
      (git/git-clone url
                     :branch branch
                     :dir dir))))

(defn checkout [repo id]
  (log/debug "Checking out" id "from repo" repo)
  (git/git-checkout repo {:name id}))

(defn ^:deprecated clone+checkout
  "Clones the repo, then performs a checkout of the given id"
  [{:keys [branch commit-id ref] :as opts}]
  (let [repo (clone opts)]
    (when-let [id-or-branch (or commit-id ref (prefix-origin branch))]
      (checkout repo id-or-branch))
    repo))

(defn delete-repo
  "Deletes the previously checked out local repo"
  [repo]
  (log/debug "Deleting repository" repo)
  (-> repo
      (.getRepository)
      (.getWorkTree)
      (u/delete-dir)))
